\name{RODBC-internal}
\alias{odbcColData}
\alias{odbcNumCols}
\alias{odbcUpdate}
\alias{odbcCaseFlag}
\alias{odbcSpecialColumns}
\alias{sqltablecreate}
\alias{sqlwrite}
\alias{odbcTableExists}
\alias{odbcTypeInfo}
\alias{odbcClearResults}
\alias{odbcValidChannel}

\title{Internal RODBC functions}
\description{
  Internal RODBC functions: not exported from the \pkg{RODBC} namespace
}
\usage{
odbcColData(channel)
odbcNumCols(channel)

odbcUpdate(channel, query, data, params, test = FALSE,
           verbose = FALSE, nastring = NULL)
odbcCaseFlag(channel)
odbcSpecialColumns(channel, table)
sqlwrite(channel, tablename, mydata, test = FALSE, fast = TRUE,
         nastring = NULL, verbose = FALSE)
sqltablecreate(tablename, coldata = NULL, colspecs, keys = -1)
odbcTableExists(channel, tablename, abort = TRUE, forQuery = TRUE)
odbcTypeInfo(channel, type)
odbcClearResults(channel)
odbcValidChannel(channel)
}
\keyword{internal}

\eof
\name{RODBC}
\alias{RODBC}
\title{ODBC Database Connectivity}
\description{
  \code{RODBC} implements ODBC database connectivity with compliant
  databases where drivers exist on the host system.
}
\details{
  Two groups of commands are provided.  The mainly internal \code{odbc*}
  commands implement relatively low level access to the odbc functions
  of similar name.  \code{sql*} commands are higher level constructs to
  read, save, copy and manipulate data between data frames and sql
  tables.  Many connections can be open at once to any combination of
  dsn/hosts.

  The functions try to cope with the peculiar way the Excel ODBC
  driver handles table names.  However, SQL expects both table and
  column names to be alphanumeric plus \code{_}, and RODBC does not
  support vendor extensions.  Most of the functions will drop
  other characters from table and column names.

  \code{options(dec)} can be used to set the decimal point to be
  used when reading numbers from character data on the database: the
  default is taken from the current locale by \code{\link{Sys.localeconv}}.
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{data(USArrests) # R < 2.0.0 only
channel <- odbcConnect("test")
sqlSave(channel, USArrests, rownames = "State", verbose = TRUE)
sqlQuery(channel, paste("select State, Murder from USArrests",
                        "where Rape > 30 order by Murder"))
sqlFetch(channel, "USArrests", rownames = "State")
sqlDrop(channel, "USArrests")
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{RODBCtables}
\alias{sqlPrimaryKeys}
\alias{sqlColumns}
\alias{sqlDrop}
\alias{sqlClear}

\title{Operations on Tables in ODBC databases}
\description{
  Operations on tables in ODBC databases.
}
\usage{
sqlClear(channel, sqtable, errors = TRUE)

sqlDrop(channel, sqtable, errors = TRUE)

sqlColumns(channel, sqtable, errors = FALSE, as.is = TRUE,
           special = FALSE)

sqlPrimaryKeys(channel, sqtable, errors = FALSE, as.is = TRUE)
}
\arguments{
  \item{channel}{connection object as returned by \code{\link{odbcConnect}}.}
  \item{sqtable}{character: a database table name accessible from the
    connected dsn.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
  \item{special}{return only the column(s) needed to specify a row
    uniquely.  Depending on the database, there might be none.}
}
\details{
  \code{sqlClear} deletes the content of the table \code{sqtable}.  No
  confirmation is requested.

  \code{sqlDrop} removes the table \code{sqtable}.  No confirmation
  is requested.

  \code{sqlColumns} and \code{sqlPrimaryKeys} return information as data
  frames.  The column names are not constant across ODBC versions so the
  data should be accessed by column number.  The argument \code{special}
  to \code{sqlColumns} returns the columns needed to specify a row
  uniquely.  This is intended to form the basis of a WHERE clause for
  updates (see \code{\link{sqlUpdate}}).
}
\value{
  A data frame on success, or character/numeric on error depending on the
  \code{errors} parameter.  If no data is returned, either a zero-row
  data frame or an error. (For example, if there are no primary keys or
  special column(s) in this table
  an empty data frame is returned, but if primary keys are not supported
  by the DBMS, an error code results.)
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlQuery}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{## example results from MySQL
data(USArrests) # R < 2.0.0 only
channel <- odbcConnect("test")
sqlDrop(channel, "USArrests", errors = FALSE) # precautionary
sqlSave(channel, USArrests)
sqlColumns(channel, "USArrests")
sqlColumns(channel, "USArrests", special = TRUE)
sqlPrimaryKeys(channel, "USArrests")
sqlColumns(channel, "USArrests")
##   Table_cat Table_schema Table_name Column_name Data_type Type_name
## 1                         USArrests    rownames        12   varchar
## 2                         USArrests      murder         8    double
## 3                         USArrests     assault         4   integer
## 4                         USArrests    urbanpop         4   integer
## 5                         USArrests        rape         8    double
##   Column_size Buffer_length Decimal_digits Num_prec_radix Nullable Remarks
## 1         255           255           <NA>           <NA>        0
## 2          22             8             31             10        1
## 3          11             4              0             10        1
## 4          11             4              0             10        1
## 5          22             8             31             10        1
sqlColumns(channel, "USArrests", special = TRUE)
##   Scope Column_name Data_type Type_name Precision Length Scale
## 1     2    rownames        12   varchar         0      0     0
##   Pseudo_column
## 1             1
sqlPrimaryKeys(channel, "USArrests")
##   Table_qualifer Table_owner Table_name Column_name Key_seq Pk_name
## 1           <NA>        <NA>  USArrests    rownames       1 PRIMARY
sqlClear(channel, "USArrests")
sqlDrop(channel, "USArrests")
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{odbc}
\alias{odbcQuery}
\alias{odbcTables}
\alias{odbcColumns}
\alias{odbcPrimaryKeys}
\alias{odbcFetchRows}

\title{Low-level ODBC functions}
\description{
  \R functions which talk directly to the ODBC interface.
}
\usage{
odbcTables(channel)

odbcColumns(channel, table)
odbcPrimaryKeys(channel, table)

odbcQuery(channel, query)
odbcFetchRows(channel, max = 0, buffsize = 1000, nullstring = NA,
              believeNRows = TRUE) 
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}
  \item{query}{any valid SQL statement}
  \item{table}{a database table name accessible from the connected dsn.
    This can be either a character string or an (unquoted) symbol.}
  \item{max}{limit on the number of rows to fetch, with 0 indicating no limit.}
  \item{buffsize}{the number of records to be transferred at a time.}
  \item{nullstring}{
    character string to be used when reading \code{SQL_NULL_DATA} items
    from the database.
  }
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  Not true for Oracle, apparently.}
}
\details{
  \code{odbcFetchRows} returns a data frame of the pending
  rowset in \code{$data} limited to \code{max} rows if
  \code{max} is greater than 0.  \code{buffsize} may be increased from the
  default of 1000 rows for increased performance in a large dataset.
  This only has an effect with servers that do not return the number of rows
  affected by a query e.g. MS Access, MS SQLServer.
}
\value{
  Most return \code{1} on success and \code{-1} on failure,
  indicating that a message is waiting for \code{\link{odbcGetErrMsg}}.
  \code{odbcFetchRows} may return \code{-2} indicating \code{"No Data"},
  the message that would be returned by \code{\link{odbcGetErrMsg}}.
}
\seealso{
  \code{\link{sqlQuery}}, \code{\link{odbcConnect}},
  \code{\link{odbcGetErrMsg}}.
}
\author{
  Michael Lapsley and Brian Ripley
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcClose}
\alias{odbcClose}
\alias{odbcCloseAll}
\alias{close.RODBC}
\title{ODBC Close Connections}
\description{
  Close connections to ODBC databases.
}
\usage{
odbcClose(channel)

\method{close}{RODBC}(con, \dots)

odbcCloseAll()
}
\arguments{
  \item{channel, con}{RODBC connection object returned by \code{odbcConnect}.}
  \item{\dots}{additional arguments passed from the generic.}
}
\details{
  \code{odbcClose} cleans up and frees resources. It is also the method for the
  generic function \code{close}.

  \code{odbcCloseAll} closes all open channels (amongst the first 100
  used in the session)

  Channels are closed at the end of an \R session, and may also be
  closed by garbage collection if no object refers to them.
}
\value{
  Function \code{odbcClose} returns invisibly a logical indicating if it
  succeeded.
}
\author{
  Michael Lapsley, Brian Ripley
}
\seealso{
  \code{\link{odbcConnect}}
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcConnect}
\alias{odbcConnect}
\alias{odbcDriverConnect}
\alias{odbcReConnect}
\alias{print.RODBC}
#ifdef windows
\alias{odbcConnectAccess}
\alias{odbcConnectDbase}
\alias{odbcConnectExcel}
#endif
\title{ODBC Open Connections}
\description{
  Open connections to ODBC databases.
}
\usage{
odbcConnect(dsn, uid = "", pwd = "", case = "nochange",
            believeNRows = TRUE)

odbcDriverConnect(connection = "", case = "nochange",
                  believeNRows = TRUE)

odbcReConnect(channel, case, believeNRows)
#ifdef windows

odbcConnectAccess(access.file, uid = "", pwd = "")
odbcConnectDbase(dbf.file)
odbcConnectExcel(xls.file)
#endif
}
\arguments{
  \item{dsn}{character string. A registered data source name.}
  \item{uid, pwd}{UID and password for authentication (if required).}
  \item{connection}{character string. See your ODBC documentation for the
    format.}
  \item{case}{Controls case changes for different RDBMS engines.}
  \item{channel}{RODBC connection object returned by \code{odbcConnect}.}
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  Not true for Oracle and Sybase, apparently.}
#ifdef windows
  \item{access.file, dbf.file, xls.file}{file of an appropriate type.}
#endif
}
\details{
  \code{odbcConnect} establishes a connection to the dsn, and
  \code{odbcDriverConnect} allows a more flexible specification via a
  connection string.  \code{odbcConnect} uses the connection string\cr
  \code{"DSN=dsn;UID=uid;PWD=pwd"}, omitting the last two comments if
  they are empty.  See the examples for other uses of connection strings.

#ifdef windows
  Under the Windows GUI, specifying an incomplete \code{connection}, for
  example the default \code{""}, will bring up a dialog box to complete the
  information required.  (This does not work from \code{Rterm.exe}
  unless a driver is specified, a Windows restriction.)

#endif
  For databases that translate table and column names the case must be
  set as appropriate.  Allowable values are \code{"nochange"},
  \code{"toupper"} and \code{"tolower"} as well as the names of databases
  where the behaviour is known to us (currently \code{"mysql"} (which
  maps to lower case on Windows but not on Linux),
  \code{"postgresql"} (lower), \code{"oracle"} (upper) and
  \code{"msaccess"} (nochange)).

  Function \code{odbcReConnect} re-connects to a database using the
  settings of an existing (and presumably now closed) channel object.
  Arguments \code{case} and \code{believeNRows} are taken from the
  object, but can be overridden by supplying those arguments.
#ifdef windows

  \code{odbcConnectAccess}, \code{odbcConnectDbase} and
  \code{odbcConnectExcel} are convenience wrappers to generate
  connection strings for those file types.  The files given can be
  relative to the \R working directory or absolute paths (and it seems
  that the drivers will also look relative to the user's home
  directory).  \bold{Note:} they will only work with English versions of
  the Microsoft drivers, which may or may not be installed in other
  locales.  The file name can be omitted which will bring up a dialog
  box to search for a file.
#endif
}
\value{
  A non-negative integer which is used as handle if no error occurred,
  -1 otherwise. A successful return has class \code{"RODBC"}, and
  attribute \code{"connection.string"} giving the full ODBC connection string.
}
\author{
  Michael Lapsley, Brian Ripley
}
\seealso{
  \code{\link{odbcClose}}, \code{\link{sqlQuery}}, \code{\link{odbcGetInfo}}
}
\examples{\dontrun{
#ifdef unix
# MySQL
channel <- odbcConnect("test", uid="ripley", pwd="secret")
# PostgreSQL
channel <- odbcConnect("pg", uid="ripley", pwd="secret", case="postgresql")
#endif
#ifdef windows
# interactive specification under RGui
channel <- odbcDriverConnect("")

# MySQL on Windows -- MySQL maps to lower case on Windows only
channel <- odbcConnect("testdb", uid="ripley", case="tolower")

# Access
channel <- odbcConnect("testacc") # if this was set up as a DSN
channel2 <- odbcConnectAccess("test.mdb", uid="ripley")

# Excel
channel <- odbcConnect("bdr.xls") # if this was set up as a DSN
channel2 <-
 odbcDriverConnect("DRIVER=Microsoft Excel Driver (*.xls);DBQ=C:\\bdr\\hills.xls")
channel3 <- odbcConnectExcel("hills.xls")
#endif

# re-connection
odbcCloseAll()
channel <- odbcReConnect(channel) # must re-assign as the data may well change
}}
\keyword{IO}
\keyword{database}

\eof
\name{odbcErrors}
\alias{odbcGetErrMsg}
\alias{odbcClearError}

\title{RODBC Error-Handling Utility Functions}
\description{
  Utility functions for examining and clearing error messages which have
  been collected during low-level ODBC calls.
}
\usage{
odbcGetErrMsg(channel)
odbcClearError(channel)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}  
}
\value{
  \code{odbcGetErrMsg} returns a (possibly zero-length)
  character vector of pending messages.

  \code{odbcClearError} returns nothing, invisibly.
}
\seealso{
  \code{\link{odbcQuery}}, \code{\link{odbcConnect}}
}
\author{
  Michael Lapsley
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcGetInfo}
\alias{odbcGetInfo}
\title{Request Information on an ODBC Connection}
\description{
  Request information on an ODBC connection.
}
\usage{
odbcGetInfo(channel)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}
}
\value{
  A named character string giving information on the database and ODBC driver
  in use on the connection \code{channel}.
}
\author{
  Brian Ripley
}
\examples{\dontrun{odbcGetInfo(channel) # under Windows XP
## MySQL returned
             DBMS_Name               DBMS_Ver        Driver_ODBC_Ver
               "MySQL"           "4.0.20a-nt"                "03.51"
      Data_Source_Name            Driver_Name             Driver_Ver
             "testdb3"          "myodbc3.dll"             "03.51.09"
              ODBC_Ver            Server_Name
          "03.52.0000" "localhost via TCP/IP"
## MS Access returned
       DBMS_Name         DBMS_Ver  Driver_ODBC_Ver Data_Source_Name
        "ACCESS"     "04.00.0000"          "03.51"        "testacc"
     Driver_Name       Driver_Ver         ODBC_Ver      Server_Name
  "odbcjt32.dll"     "04.00.6304"     "03.52.0000"         "ACCESS"
## MSDE 2000 returned
             DBMS_Name               DBMS_Ver        Driver_ODBC_Ver
"Microsoft SQL Server"           "08.00.0760"                "03.52"
      Data_Source_Name            Driver_Name             Driver_Ver
                "MSDE"         "SQLSRV32.DLL"           "03.85.1117"
              ODBC_Ver            Server_Name
          "03.52.0000"                  "AUK"
}}
\keyword{IO}
\keyword{database}

\eof
\name{setSqlTypeInfo}
\alias{setSqlTypeInfo}
\alias{getSqlTypeInfo}
\title{Specify a Mapping of R Types to DBMS Types}
\description{
  Specify or retrieve a mapping of \R types to DBMS datatypes.
}
\usage{
setSqlTypeInfo(driver, value)

getSqlTypeInfo(driver)
}
\arguments{
  \item{driver}{A character string specifying the driver as returned by
    \code{\link{odbcGetInfo}}.  Optional for \code{getSqlTypeInfo}.}
  \item{value}{A named list with character values.  This should have
    names \code{"double"}, \code{"integer"}, \code{"character"} and
    \code{"logical"}.}
}
\details{
  This information is used by \code{\link{sqlSave}} if it creates a
  table in the DBMS.

  The types chosen should be nullable to allow \code{NA}s to be
  represented.  (Bit and boolean types generally are not.)
}
\value{
  For \code{setSqlTypeInfo} none.

  For \code{getSqlTypeInfo} with an argument, a named list.  Without an
  argument, a data frame.
}
\author{
  Brian Ripley
}
\seealso{
  \code{\link{sqlTypeInfo}}, \code{\link{sqlSave}}.
}
\examples{\dontrun{
getSqlTypeInfo()
getSqlTypeInfo("MySQL")
setSqlTypeInfo("SQLServer",
               list(double="float", integer="int",
                    character="varchar(255)", logical="varchar(5)"))
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlCopy}
\alias{sqlCopy}
\alias{sqlCopyTable}

\title{ODBC Copy}
\usage{
sqlCopy(channel, query, destination, destchannel = channel,
        verbose = FALSE, errors = TRUE, \dots)

sqlCopyTable(channel, srctable, desttable, destchannel = channel,
             verbose = FALSE, errors = TRUE)
}
\arguments{
  \item{channel, destchannel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{query}{any valid SQL statement}
  \item{destination, srctable, desttable}{character: a
    database table name accessible from the connected dsn.}
  \item{verbose}{Display statements as they are sent to the server?}
  \item{errors}{if TRUE halt and display error, else return \code{-1}.}
  \item{\dots}{additional arguments to be passed to \code{\link{sqlSave}}.}
}
\description{
  Functions to copy tables or result sets from one database to another.
}
\details{
  \code{sqlCopy} as is like \code{\link{sqlQuery}},
  but saves the output of \code{query} in table \code{destination} on
  channel \code{destchannel}.

  \code{sqlCopyTable}
  copies the structure of \code{srctable} to \code{desttable} on dsn
  \code{destchannel}.  This is within the limitations of the ODBC lowest
  common denominator.  More precise control is possible via
  \code{sqlQuery}.
}
\value{
  See \code{\link{sqlGetResults}}.
}
\seealso{
  \code{\link{sqlQuery}}, \code{\link{sqlSave}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{\dontrun{## example for a l/casing DBMS
sqlSave(channel, USArrests, rownames = "state")
query <- paste("select state, murder from usarrests",
               "where rape > 30", "order by murder")
sqlCopy(channel, query, "HighRape", rownames = FALSE)
sqlFetch(channel, "HighRape", rownames = "state", max = 5)
sqlDrop(channel, "HighRape")
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlFetch}
\alias{sqlFetch}
\alias{sqlFetchMore}

\title{Reading Tables from ODBC Databases}
\description{
  Read a table of an ODBC database into a data frame.
}
\usage{
sqlFetch(channel, sqtable, ..., colnames = FALSE, rownames = TRUE)

sqlFetchMore(channel, ..., colnames = FALSE, rownames = TRUE)
}
\arguments{
  \item{channel}{connection handle returned by \code{\link{odbcConnect}}.}
  \item{sqtable}{a database table name accessible from the connected dsn.
    This should be either a character string or a character vector of
    length 1.}
  \item{\dots}{additional arguments to be passed to
    \code{\link{sqlGetResults}}.}
  \item{colnames}{logical: retrieve column names from first row of table?}
  \item{rownames}{either logical or character.
    If logical, retrieve row names from the first column
    (\code{rownames}) in the table?  If character, the column name to
    retrieve them.}
}
\details{
  \code{sqlFetch} retrieves the the entire contents of the table
  \code{sqtable}.  Rownames and column names are restored as indicated.
  \code{sqlFetchMore} will retrieve further results from the query
  (provided there has been no other ODBC query on that channel in the
  meantime).

  It tries to cope with the peculiar way the Excel ODBC driver handles
  table names, and to quote Access table names which contain spaces.

  Useful additional parameters to pass to \code{\link{sqlGetResults}} include
  \describe{
    \item{\code{max}:}{limit on the number of rows to fetch, with
      0 (the default) indicating no limit.}
    
    \item{\code{nullstring}:}{character string to be used when reading
      \code{SQL_NULL_DATA} character items from the database.}
    
    \item{\code{na.strings}:}{character string(s) to be mapped to
      \code{NA} when reading character data.}
    
    \item{\code{as.is}:}{as in \code{\link{sqlGetResults}}.}
    
    \item{\code{dec}:}{The character for the decimal place to be assumed
      when converting character columns to numeric.}
  }
}
\value{
  A data frame on success, or a character or numeric error code (see
  \code{\link{sqlQuery}}).
}
\note{
  If the table name desired is not a valid SQL name (alphanumeric plus
  \code{_}), use \code{\link{sqlQuery}} with whatever quoting mechanism
  your DBMS vendor provides (e.g. \code{[ ]} on Microsoft products and
  backticks on recent versions of MySQL).
}
\seealso{
  \code{\link{sqlSave}}, \code{\link{sqlQuery}},
  \code{\link{odbcConnect}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{
data(USArrests) # R < 2.0.0 only
channel <- odbcConnect("test")
sqlSave(channel, USArrests, verbose = TRUE)
sqlFetch(channel, "USArrests") # get the lot
sqlFetch(channel, "USArrests", max=20)
sqlFetchMore(channel, max=20)
sqlFetchMore(channel) # get the rest
sqlDrop(channel, "USArrests") 
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlQuery}
\alias{sqlQuery}
\alias{sqlGetResults}

\title{Query an ODBC Database}
\description{
  Submit an SQL query to an ODBC database, and retrieve the results.
}
\usage{
sqlQuery(channel, query, errors = TRUE, ...)

sqlGetResults(channel, as.is = FALSE,  errors = FALSE,
              max = 0, buffsize = 1000,
              nullstring = NA, na.strings = "NA",
              believeNRows = TRUE, dec = getOption("dec"))
}
\arguments{
  \item{channel}{connection handle as returned by \code{\link{odbcConnect}}.}
  \item{query}{any valid SQL statement}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{\dots}{additional arguments to be passed to \code{sqlGetResults}.}

  \item{as.is}{which (if any) character columns should be converted,
    as in \code{\link{read.table}}?  See the details.}
  \item{max}{limit on the number of rows to fetch, with 0 indicating no limit.}
  \item{buffsize}{the number of rows to be transferred at a time.}
  \item{nullstring}{
    character string to be used when reading \code{SQL_NULL_DATA}
    character items from the database.
  }
  \item{na.strings}{
    character string(s) to be mapped to \code{NA} when reading character
    data.
  }
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  This might already be set to false when
    the channel was opened, and can that setting cannot be overriden.}
  \item{dec}{The character for the decimal place to be assumed when
    converting character columns to numeric.}
}
\details{
  \code{sqlQuery} is the workhorse function of RODBC.  It sends the SQL
  statement \code{query} to the server, using connection \code{channel}
  returned by \code{\link{odbcConnect}}.

  SQL beginners should note that the term 'Query' includes
  any valid SQL statement including table creation, alteration, updates
  etc as well as SELECTs. The \code{sqlQuery} command is a convenience
  wrapper that calls first \code{\link{odbcQuery}} and then
  \code{sqlGetResults}. If finer-grained control is needed, for example
  over the number of rows fetched, these functions should be called
  directly or additional arguments passed to \code{sqlQuery}.

  \code{sqlGetResults} is a mid-level function.  It should be called
  after a call to \code{sqlQuery} or \code{\link{odbcQuery}} and used to
  retrieve waiting results into a data frame.  Its main use is with
  \code{max} set to non-zero when it will retrieve the result set in
  batches with repeated calls.  This is useful for very large result sets
  which can be subjected to intermediate processing.

  Where possible \code{sqlGetResults} transfers data directly: this
  happens for \code{double}, \code{real}, \code{integer} and
  \code{smallint} columns in the table.  All other SQL data types are
  converted to character strings by the ODBC interface.  If the
  \code{as.is} is true for a column, it is returned as character.
  Otherwise (where detected) \code{date}, \code{datetime} and
  \code{timestamp} values are converted to \code{"Date"} and
  \code{"POSIXct"} values respectively.  (Some drivers seem to confuse times
  with dates, so times may get converted too.)  Other types are
  converted by \R using \code{\link{type.convert}}.  When
  character data are to be converted to numeric data, the setting of
  \code{options("dec")} to map the character used up the ODBC driver in
  setting decimal points---this is set to a locale-specific value
  when RODBC is initialized if it is not already set.
  
  Using \code{buffsize} will yield a marginal increase in speed if set
  to no less than the maximum number of rows where \code{believeNRows =
    FALSE}.
}
\value{
  A data frame (possibly with 0 rows) on success.
  If \code{errors = TRUE}, a character vector of error message(s),
  otherwise error code \code{-1} (general, call
  \code{\link{odbcGetErrMsg}} for details) or \code{-2} (no data, which
  may not be an error as some SQL commands do return no data).
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcQuery}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{
data(USArrests) # R < 2.0.0 only
channel <- odbcConnect("test")
sqlSave(channel, USArrests, rownames = "State", verbose = TRUE)
# options(dec=".") # optional, if DBMS is not locale-aware
## note case of State, Murder, rape are DBMS-dependent.
sqlQuery(channel, paste("select State, Murder from USArrests",
                        "where Rape > 30 order by Murder"))
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlSave}
\alias{sqlSave}
\alias{sqlUpdate}

\title{Write a Data Frame to a Table in an ODBC Database}
\description{
  Write or update a table in an ODBC database.
}
\synopsis{
sqlSave(channel, dat, tablename = NULL, append = FALSE,rownames = TRUE,
        colnames = FALSE, verbose = FALSE, oldstyle = FALSE,
        safer = TRUE, addPK = FALSE, typeInfo, varTypes, ...)

sqlUpdate(channel, dat, tablename = NULL, index = NULL,
          verbose = FALSE, test = FALSE, nastring = NULL, fast = TRUE)
}
\usage{
sqlSave(channel, dat, tablename = NULL, append = FALSE,
        rownames = TRUE, colnames = FALSE,
        verbose = FALSE, oldstyle = FALSE,
        safer = TRUE, addPK = FALSE, typeInfo, varTypes,
        fast = TRUE, test = TRUE, nastring = NULL)

sqlUpdate(channel, dat, tablename = NULL, index = NULL,
          verbose = FALSE, test = FALSE,
          nastring = NULL, fast = TRUE)
}
\arguments{
  \item{channel}{connection handle returned by \code{\link{odbcConnect}}.}
  \item{dat}{a data frame.}
  \item{tablename}{character: a database table name accessible from
    the connected dsn.  If missing, the name of \code{dat}.}
  \item{index}{character.  Name(s) of index column(s) to be used.}
  \item{append}{logical.  Should data be appended to an existing table?}
  \item{rownames}{either logical or character.
    If logical, save the row names as the first column \code{rownames}
    in the table?  If character, the column name under which to save
    the rownames.}
  \item{colnames}{logical: save column names as the first row of table?}
  \item{verbose}{display statements as they are sent to the server?}

  \item{oldstyle}{logical. If false, attempt to use \code{sqlTypeInfo}
    to choose the types of columns when a table has to be created.  If
    true, create all columns as \code{varchar(255)}.}
  \item{safer}{logical. If true, create a non-existing table but only
    allow appends to an existing table. If false, allow \code{sqlSave}
    to attempt to delete all the rows of an existing table, or to drop it.}
  \item{addPK}{logical.  Should rownames (if included) be specified as a
    primary key?}
  \item{typeInfo}{optional list of DBMS datatypes.  Should have elements
    named \code{"character"}, \code{"double"} and \code{"integer"}.}
  \item{varTypes}{an optional named character vector giving the DBMSs
    datatypes to be used for some (or all) of the columns if a table is
    to be created.}
  \item{fast}{logical.  If false, write data a row at a time.  If true,
    use a parametrized INSERT INTO / UPDATE query to write all the data
    in one operation.}
  \item{test}{show what would be done, only.}
  \item{nastring}{character string to be used for writing \code{NA}s
    to the database.  The default, \code{NULL}, attempts to write
    a missing value as a database null.}
%  \item{\dots}{any additional arguments are passed to \code{\link{sqlwrite}}.}
}
\details{
  \code{sqlSave} saves the data frame \code{dat} in the table
  \code{tablename}.  If the table exists and has the appropriate
  structure it is used, or else it is created anew.  If a new table is
  created, column names are remapped by removing any characters which
  are not alphanumeric or \code{_}, and the types are selected by
  consulting arguments \code{varTypes} and \code{typeInfo}, then looking
  the driver up in the database used by \code{\link{getSqlTypeInfo}} or
  failing that by interrogating \code{\link{sqlTypeInfo}}.

  If \code{rownames = TRUE} the first column of the table will be the
  row labels with colname \code{rowname}: \code{rownames} can also be a
  string giving the desired column name (see example). \code{colnames}
  copied the column names into row 1.  This is intended for cases where
  case conversion alters the original column names and it is desired
  that they are retained.  Note that there are drawbacks to this
  approach: it presupposes that the rows will be returned in correct
  order; not always valid.  It will also cause numeric rows to be
  returned as factors.

  Argument \code{addPK = TRUE} causes the rownames to be marked as a
  primary key.  This is usually a good idea, and may allow updates to be
  done.  However, some DBMSs (e.g. Access) do not support primary keys,
  and the PostgreSQL ODBC driver has generated internal memory
  corruption if this option is used.
  
  \code{sqlUpdate} updates the table where the rows already exist. Data
  frame \code{dat} should columns with names that map to (some of) the
  columns in the table.  It also needs to contain the column(s)
  specified by \code{index} which together identify the rows to be
  updated.  If \code{index = NULL}, the function tries to identify such
  rows.  First it looks for a primary key in the data frame, then for
  the column(s) that the database regards as the optimal for defining a
  row uniquely (these are returned by
  \code{\link{sqlColumns}(\dots, special=TRUE)}).  If
  there is no such column the rownames are used provided they are stored
  as column \code{"rownames"} in the table.

  The value of \code{nastring} is used for all the columns and no
  attempt is made to check if the column is nullable.  For all but the
  simplest applications it will be better to prepare a data frame with
  non-null missing values already substituted.

  If \code{fast = FALSE} all data is sent as character strings.
  If \code{fast = TRUE}, integer and double vectors are sent as types
  \code{SQL_C_SLONG} and \code{SQL_C_DOUBLE} respectively.  Some drivers
  seem to require \code{fast = FALSE} to send other types,
  e.g. \code{datetime}.
}
\section{Warning}{
  \code{sqlSave(safer = FALSE)} uses the 'great white shark'
  method of testing tables (bite it and see).  The logic will
  unceremoniously DROP the table and create it anew with its own choice of
  column types in its attempt to find a writable solution.
  \code{test = TRUE} will not necessarily predict this behaviour.
  Attempting to write indexed columns or writing to pseudo-columns are
  less obvious causes of failed writes followed by a DROP.  If your table
  structure is precious to you back it up.  
}
\value{
  1 invisibly for success (and failures cause errors).
}
\seealso{
  \code{\link{sqlFetch}}, \code{\link{sqlQuery}},
  \code{\link{odbcConnect}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{
data(USArrests) # R < 2.0.0 only
channel <- odbcConnect("test")
sqlSave(channel, USArrests, rownames = "state", addPK=TRUE)
sqlFetch(channel, "USArrests", rownames = "state") # get the lot
foo <- cbind(state=row.names(USArrests), USArrests)[1:3, c(1,3)]
foo[1,2] <- 222
sqlUpdate(channel, foo, "USArrests")
sqlFetch(channel, "USArrests", rownames = "state", max = 5)
sqlDrop(channel, "USArrests") 
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlTables}
\alias{sqlTables}

\title{List Tables on an ODBC Database}

\description{
  List the tables on an ODBC database.
}
\usage{
sqlTables(channel, errors = FALSE, as.is = TRUE)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
}
\value{
  A data frame on success, or character/numeric
  on error depending on the \code{errors} parameter. (See
  \code{\link{sqlGetResults}} for further details.)

  The column names depend on the database, containing a column
  \code{table_name} in some mixture of cases (see the examples).
}
\seealso{
  \code{\link{sqlGetResults}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{
channel <- odbcConnect("test")
sqlTables(channel, "USArrests")
## MySQL example
##   Table_qualifer Table_owner Table_name Table_type     Remarks
##1                             usarrests      TABLE MySQL table
## Microsoft Access example
##      TABLE_CAT TABLE_SCHEM        TABLE_NAME   TABLE_TYPE REMARKS
##1 C:\\bdr\\test        <NA> MSysAccessObjects SYSTEM TABLE    <NA>
##2 C:\\bdr\\test        <NA>          MSysACEs SYSTEM TABLE    <NA>
##3 C:\\bdr\\test        <NA>       MSysObjects SYSTEM TABLE    <NA>
##4 C:\\bdr\\test        <NA>       MSysQueries SYSTEM TABLE    <NA>
##5 C:\\bdr\\test        <NA> MSysRelationships SYSTEM TABLE    <NA>
##6 C:\\bdr\\test        <NA>             hills        TABLE    <NA>
##7 C:\\bdr\\test        <NA>         USArrests        TABLE    <NA>
close(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlTypeInfo}
\alias{sqlTypeInfo}

\title{Request Information about DataTypes in an ODBC Database}
\description{
  Request information about datatypes in an ODBC database
}
\usage{
sqlTypeInfo(channel, type = "all", errors = TRUE, as.is = TRUE)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{type}{The types of columns about which information is
    requested. Possible values are \code{"all", "char", "varchar", "real",
      "float", "double", "integer", "smallint", "timestamp"}.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
}
\details{
  \code{sqlTypeInfo} attempts to find the types of columns the database
  supports.  Not all ODBC drivers support this.  Where it is supported,
  it is used to decide what column types to create when creating a new
  table in the database.
}
\value{
  A data frame on success, or character/verbose
  on error depending on the \code{errors} parameter.  See
  \code{\link{sqlGetResults}} for further details.
}
\seealso{
  \code{\link{sqlGetResults}}, \code{\link{odbcGetInfo}}
}
\author{
  Brian Ripley
}
\keyword{IO}
\keyword{database}

\eof
